// NinjaRMI, by Matt Welsh (mdw@cs.berkeley.edu)
// See http://www.cs.berkeley.edu/~mdw/proj/ninja for details

/*
 * "Copyright (c) 1998 by The Regents of the University of California
 *  All rights reserved."
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 */

package ninja.rmi.registry;

import java.rmi.registry.Registry;
import java.util.Hashtable;
import java.util.Enumeration;
import java.rmi.*;
import ninja.rmi.*;
import java.net.InetAddress;

/**
 * NinjaRegistryImpl is an implementation of the Registry interface, and
 * acts as a registry for remote objects.
 */
public class NinjaRegistryImpl extends NinjaRemoteObject implements Registry {

  // XXX Need to make this synchronized somehow?
  private Hashtable services;
  private InetAddress localhost;
  private String localhostname;

  public NinjaRegistryImpl() throws RemoteException {
    super((NinjaExportData)null); // Don't export ourselves yet
    NinjaExportData exportData = new NinjaExportData();
    exportData.port = Registry.REGISTRY_PORT;
    exportData.check_objid = false;
    this.exportObject(exportData); // Ok, now we're exported
    
    services = new Hashtable();
    try {
      localhost = InetAddress.getLocalHost();
      localhostname = localhost.getHostName();
    } catch (Exception e) {
      throw new RemoteException(e.getMessage());
    }
  }

  public Remote lookup(String name) throws RemoteException {
    Remote remobj = (Remote)services.get(name);
    if (remobj == null) throw new RemoteException("Can't find "+name);
    return remobj;
  }

  public void bind(String name, Remote obj) throws RemoteException {
    allowBind();
    Object oldval = services.put(name, obj);
    if (oldval != null) {
      throw new RemoteException(name + " already bound in registry");
    }
  }

  public void unbind(String name) throws RemoteException {
    allowBind();
    Object val = services.get(name);
    if (val == null) throw new RemoteException(name + " not bound");
    services.remove(name);
  }

  public void rebind(String name, Remote obj) throws RemoteException {
    allowBind();
    Object oldval = services.put(name, obj);
  }

  public String[] list() throws RemoteException {
    int n = services.size();
    String thelist[] = new String[n];
    Enumeration e = services.keys();
    n = 0;
    while (e.hasMoreElements()) {
      thelist[n] = (String)e.nextElement();
      n++;
    }
    return thelist;
  }

  private void allowBind() throws RemoteException {
    String clientname;
    try {
      Thread t = Thread.currentThread();
      if (t instanceof ninja.rmi.Reliable_ServerThread) {
        clientname = ((ninja.rmi.Reliable_ServerThread)t).getClientHost();
      } else {
	throw new RemoteException("Registry must use Reliable transport");
      }
      
      InetAddress client = InetAddress.getByName(clientname);

      // Check the name _and_ the IP address - to resolve problems with,
      // e.g., 'bhikku' and 'bhikku.cs.berkeley.edu' or multiple names/IPs
      // per host
      if (!localhostname.equals(clientname) &&
	  !localhost.equals(client)) {
	throw new RemoteException("Ninja RmiRegistry: Localhost "+localhostname+" doesn't match client host "+clientname);
      }
    } catch (Exception e) {
      throw new RemoteException(e.getMessage());
    }
  }
      

  public static void main(String args[]) {
    try {
      NinjaRegistryImpl reg = new NinjaRegistryImpl();
    } catch (Exception e) {
      System.out.println("Can't create registry: "+e.getMessage());
      System.exit(1);
    }
    System.out.println("Registry started.");
    
    // XXX Need a thread or something to sleep here?
  }
  
}
